/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.console.portlet.orchestra;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.ExoContainerContext;
import org.exoplatform.services.organization.auth.AuthenticationService;
import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.config.annotation.EventConfig;
import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.event.Event;
import org.exoplatform.webui.event.EventListener;
import org.ow2.orchestra.console.security.ACLKeyManager;
import org.ow2.orchestra.console.security.NovaACL;
import org.ow2.orchestra.console.security.SecurityService;
import org.ow2.orchestra.console.service.orchestra.OrchestraService;
import org.ow2.orchestra.console.webui.commons.NovaAction;
import org.ow2.orchestra.console.webui.commons.NovaData;
import org.ow2.orchestra.console.webui.commons.NovaLink;
import org.ow2.orchestra.console.webui.commons.UINovaGrid;
import org.ow2.orchestra.console.webui.novaview.UINovaView;
import org.ow2.orchestra.console.webui.novaview.UINovaViewManagement;
import org.ow2.orchestra.facade.def.ProcessDefinition;

/**
 * UIProcess is the view component of an orchestra process, it allow to get the
 * processes from the orchestra business service, consult processes and also to
 * perform import, undeploy, start actions on those processes.
 *
 * @author Le Jeune Goulven <goulven.le-jeune@bull.net>
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 */
@ComponentConfig(template = "app:/groovy/UIProcess.gtmpl",
                 events = {@EventConfig(listeners = UIProcess.DeleteProcessActionListener.class) })
public class UIProcess extends UINovaView {

  /**
   * @throws Exception
   */
  public UIProcess() throws Exception {
    super();
  }

  /**
   * The init method set the configurations of the grid : the different headers
   * and the possible actions. Headers displayed in default mode : process name
   * Actions in default mode : delete process
   *
   */
  @Override
  public void init() {
    final UINovaGrid grid = this.getGrid();
    // String[] HEADER = {"name", "state", "createdDate"};
    final String[] header = {
      "name"
    };
    final String[] action = {
      "DeleteProcess"
    };
    grid.setHeader(header);
    // grid.setShowAction(false);
    grid.setVisibleAction(action);
  }

  /**
   * This method allow to update the grid data. It get the processes list from
   * orchestra service and update the current grid.
   *
   */
  @Override
  public void updateData() {
    final UINovaGrid grid = this.getGrid();
    final String[] params = {"processDefinitionUUID"};
    // NovaAction deleteAction = new NovaAction("DeleteProcess","name",params);
    final OrchestraService os = this.getApplicationComponent(OrchestraService.class);

    // Get data
    List<ProcessDefinition> processes = null;
    if (this.isDetail()) {
      final ProcessDefinition p = os.getProcess(this.getParam("processDefinitionUUID"));
      processes = new ArrayList<ProcessDefinition>();
      if (p != null) {
        processes.add(p);
      }
    } else {
      processes = os.getProcesses();
      final HashMap<String, NovaLink> links = new HashMap<String, NovaLink>();

      links.put("name", new NovaLink("name", this.getFactory(), "ViewProcess", params));
      grid.setLink(links);
    }

    // create the NovaData array
    final ArrayList<NovaData> data = new ArrayList<NovaData>();
    final Iterator<ProcessDefinition> ite = processes.iterator();
    while (ite.hasNext()) {
      final ProcessDefinition process = ite.next();
      final NovaData d = new NovaData(process);
      d.set("processDefinitionUUID", process.getProcessDefinitionUUID());
      d.set("name", process.getName());
      d.set("namespaceURI", process.getNameSpace());

      this.addActions(d);

      data.add(d);
    }

    grid.setData(data);
  }

  /**
   * This method allow to add actions in data element (a process) according to
   * the connected user permissions.
   *
   * @param data element
   */
  public void addActions(final NovaData d) {
    final String[] params = {"processDefinitionUUID"};
    final NovaAction deleteAction = new NovaAction("DeleteProcess", "name", params);

    // get the security service
    final SecurityService securityService = this.getApplicationComponent(SecurityService.class);
    final ExoContainer container = ExoContainerContext.getCurrentContainer();
    final AuthenticationService authenticationService =
      (AuthenticationService) container.getComponentInstanceOfType(AuthenticationService.class);
    String userName = "";
    if (authenticationService.getCurrentIdentity() != null) {
      userName = authenticationService.getCurrentIdentity().getUsername();
    }

    final NovaACL acl = securityService.getUserACL();
    final ACLKeyManager keyManager = securityService.getACLKeyManager();

    // check if the connected user has the permission in actions
    if (acl.hasPermission(userName, keyManager.generateActionKey("orchestra", "process", "undeploy"))) {
      d.addAction(deleteAction);
    }
  }

  /**
   * This subclass is the listener of the delete process event. It get the
   * selected process and use the orchestra service to undeploy it. It also
   * close all the related views and refresh the processes list.
   */
  public static class DeleteProcessActionListener extends EventListener<UIProcess> {

    /**
     * The execute method of the listener
     *
     * @throws Exception
     */
    @Override
    public void execute(final Event<UIProcess> event) throws Exception {
      final UIProcess uip = event.getSource();
      final UINovaViewManagement viewMgt = uip.getAncestorOfType(UINovaViewManagement.class);

      /* Get the process ID */
      final String processId = event.getRequestContext().getRequestParameter(UIComponent.OBJECTID);
      final String pduuid = event.getRequestContext().getRequestParameter("processDefinitionUUID");
      //String prefix = event.getRequestContext().getRequestParameter("prefix");

      final OrchestraService os = uip.getApplicationComponent(OrchestraService.class);
      os.undeployProcess(pduuid);

      viewMgt.closeView(processId);
      viewMgt.showView("Processes");
    }
  }
}
